/**
 * XjustizTransferHandler.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.files;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.swing.TransferHandler;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * TransferHandler zum Drag-and-Drop-Support für Berechnungs-XML- und
 * Xjustiz-XML-Dateien
 * 
 * @author Peter Felix Schuster (SchusterP2)
 * @date 05.08.2014
 */
public class XmlTransferHandler extends TransferHandler {

	private static final String FEHLER_KEINE_XML_DATEI = "Keine XML-Datei!";
	private static final String FEHLER_MEHRERE_DATEIEN = "Mehr als eine oder keine Datei im Zugriff!";
	public static final String XML_SUFFIX = ".xml";
	private static final long serialVersionUID = -5507906941519939902L;
	//private Kostenrechner kostenRechner;

	/**
	 * Erzeugt einen TransferHandler
	 * 
	 */
	public XmlTransferHandler() {
		super();
		//this.kostenRechner = kostenRechner;
	}

	/**
	 * This method is called repeatedly during a drag gesture and returns true if
	 * the area below the cursor can accept the transfer, or false if the transfer
	 * will be rejected. For example, if a user drags a color over a component that
	 * accepts only text, the canImport method for that component's TransferHandler
	 * should return false.
	 * 
	 * @param support the object containing the details of the transfer, not null.
	 * @return true if the import can happen, false otherwise
	 * 
	 * @see javax.swing.TransferHandler#canImport(javax.swing.TransferHandler.TransferSupport)
	 */
	@Override
	public boolean canImport(TransferSupport support) {
		// return super.canImport(support);
		if (support.isDrop())
			support.setShowDropLocation(false);
		if (!support.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
			// System.out.println
			Kostenrechner.getLogger().info("Flavor not supported: javaFileListFlavor!");
			return false;
		}
		boolean copySupported = (COPY & support.getSourceDropActions()) == COPY;
		if (!copySupported) {
			// System.out.println
			Kostenrechner.getLogger().info("COPY not supported!");
			return false;
		}
		support.setDropAction(COPY);
		return true;
	}

	/**
	 * This method is called on a successful drop (or paste) and initiates the
	 * transfer of data to the target component. This method returns true if the
	 * import was successful and false otherwise.
	 * 
	 * Causes a transfer to occur from a clipboard or a drag and drop operation. The
	 * Transferable to be imported and the component to transfer to are contained
	 * within the TransferSupport. While the drag and drop implementation calls
	 * canImport to determine the suitability of a transfer before calling this
	 * method, the implementation of paste does not. As such, it cannot be assumed
	 * that the transfer is acceptable upon a call to this method for paste. It is
	 * recommended that canImport be explicitly called to cover this case.
	 * 
	 * Note: The TransferSupport object passed to this method is only valid for the
	 * duration of the method call. It is undefined what values it may contain after
	 * this method returns.
	 * 
	 * @param support the object containing the details of the transfer, not null.
	 * @return true if the data was inserted into the component, false otherwise
	 * @throws NullPointerException if <code>support</code> is null
	 * 
	 * @see javax.swing.TransferHandler#importData(javax.swing.TransferHandler.TransferSupport)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public boolean importData(TransferSupport support) throws NullPointerException {
		// return super.importData(support);
		if (!canImport(support)) {
			return false;
		}
		Transferable t = support.getTransferable();
		try {
			Object o = (List<File>) t.getTransferData(DataFlavor.javaFileListFlavor);
			List<File> list;
			// @SuppressWarnings("unchecked")
			// List<File> l = (List<File>) t.getTransferData(DataFlavor.javaFileListFlavor);
			if (o instanceof List<?>) {
				list = (List<File>) o;
			} else {
				return false;
			}
			if (list == null || list.size() != 1) {
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, FEHLER_MEHRERE_DATEIEN,
						new IOException(FEHLER_MEHRERE_DATEIEN));
				return false;
			}
			final File xml = list.get(0);
			if (isXml(xml) || isBerechnung(xml)) {
				new VerfahrensDatenContainer2GUI().nimmXmlDatei(xml);
				return true;
			} else {
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, FEHLER_KEINE_XML_DATEI,
						new IOException(FEHLER_KEINE_XML_DATEI));
				return false;
			}
		} catch (UnsupportedFlavorException e) {
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, "Flavor not supported. " + e.getLocalizedMessage(), e);
			return false;
		} catch (IOException e) {
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
					"Data is no longer available in the requested flavor. " + e.getLocalizedMessage(), e);
			return false;
		}
	}

	/**
	 * Die Methode dient dazu, anhand der Dateiendung zu prüfen, ob es sich um eine
	 * gespeicherte Berechnung im skktx-XML-Format handelt
	 * 
	 * @param datei das zu prüfende {@link File}
	 * @return <code>true</code>, wenn der Dateiname auf
	 *         {@link BerechnungXmlDatei#DATEIENDUNG_GANZ} endet.
	 */
	public static boolean isBerechnung(final File datei) {
		return datei.getName().toLowerCase().endsWith(BerechnungXmlDatei.DATEIENDUNG_GANZ);
	}

	/**
	 * Die Methode dient dazu, anhand der Dateiendung zu prüfen, ob es sich um eine
	 * Xjustiz-XML-Datei handelt
	 * 
	 * @param datei das zu prüfende {@link File}
	 * @return <code>true</code>, wenn der Dateiname auf {@value #XML_SUFFIX} endet.
	 */
	public static boolean isXml(final File datei) {
		return datei.getName().toLowerCase().endsWith(XML_SUFFIX);
	}
}
