/**
 * EmailUtils.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.system;

import java.awt.Desktop;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;

import javax.swing.Action;

import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Die Oberklassen für alle SchreibeEmail-Klassen hält Methoden für
 * Feedback-E-Mails bereit: {@link #schreibe(String, String)} und
 * {@link #urlEncode(String)}
 *
 * @author Peter Schuster (setrok)
 * @date 28.03.2016
 *
 */
public abstract class SchreibeEmail extends BeschriebeneAktion {

	private static final long serialVersionUID = 9027687725799709474L;
	private static final String E_MAIL_FEHLER_URI_SYNTAX = "URI-Syntax falsch: ";
	private static final String E_MAIL_FEHLER_ENCODING = "Zeichenkodierung nicht unterstützt: ";
	private static final String E_MAIL_FEHLER_TEXT = "Fehler beim Aufruf des Standard-E-Mail-Programms: ";
	private static final String E_MAIL_FEHLER_TITLE = "E-Mail-Fehler";
	/**
	 * Die E-Mail-Adresse des Programmierers als String.
	 */
	protected static final String PETER_EMAIL = "peter@kostentenor.de";//"webmaster@kostentenor.de"
	/**
	 * Die URI-Einleitung für die Betreffzeile als String.
	 */
	protected static final String SUBJECT2 = "?subject=";
	/**
	 * Die URI-Einleitung für den E-Mail-Text als String.
	 */
	protected static final String BODY2 = "&BODY=";
	/**
	 * Die URI-Einleitung für E-Mails als String.
	 */
	protected static final String MAILTO = "mailto:";

	/**
	 * Ein doppelter Zeilenumbruch als String.
	 */
	protected static final String NEUER_ABSATZ = "\n\n";

	/**
	 * Konstruktor.
	 * 
	 * @param name             {@link Action#NAME}
	 * @param shortDescription {@link Action#SHORT_DESCRIPTION}
	 * @param mnemonic         {@link Action#MNEMONIC_KEY}
	 * @param actionCommandKey {@link Action#ACTION_COMMAND_KEY}
	 */
	protected SchreibeEmail(String name, String shortDescription, int mnemonic, String actionCommandKey) {
		super(name, shortDescription, mnemonic, actionCommandKey);
	}

	/**
	 * emailSchreiben lässt eine Beschwerde-E-Mail im Standard-E-Mail-Client
	 * schreiben.
	 * 
	 * @param subject die Betreffzeile der E-Mail als String
	 * @param body    der Inhalt der E-Mail als String
	 * 
	 * @url "http://stackoverflow.com/questions/12449654/clickable-email-address-as-a-jlabel-in-java"
	 * @url "http://www.2ality.com/2010/12/simple-way-of-sending-emails-in-java.html"
	 */
	protected final void schreibe(final String subject, final String body) {
		try {
			final URI mailtoURI = new URI(
					MAILTO + PETER_EMAIL + SUBJECT2 + urlEncode(subject) + BODY2 + urlEncode(body));
			// Launches the mail composing window of the user default mail
			// client (Dateien mitschicken geht wohl nicht...)
			Desktop.getDesktop().mail(mailtoURI);
			// den Browser aufzurufen dauert länger:
			// Desktop.getDesktop().browse(mailtoURI);
		} catch (UnsupportedEncodingException uee) {
			FehlerDialog.zeigeFehler(E_MAIL_FEHLER_TITLE,
					E_MAIL_FEHLER_TEXT + E_MAIL_FEHLER_ENCODING + uee.getLocalizedMessage(), uee);
		} catch (URISyntaxException use) {
			FehlerDialog.zeigeFehler(E_MAIL_FEHLER_TITLE,
					E_MAIL_FEHLER_TEXT + E_MAIL_FEHLER_URI_SYNTAX + use.getLocalizedMessage(), use);
		} catch (IOException ioe) {
			FehlerDialog.zeigeFehler(E_MAIL_FEHLER_TITLE, E_MAIL_FEHLER_TEXT + ioe.getLocalizedMessage(), ioe);
		}
	}

	/**
	 * Die Methode dient dazu, zum E-Mail-Versand eine Zeichenkette URL-konform zu
	 * kodieren.
	 * 
	 * @param str die zu kodierende Zeichenkette
	 * @url "http://www.2ality.com/2010/12/simple-way-of-sending-emails-in-java.html"
	 * @return die kodierte Zeichenkette
	 */
	protected final String urlEncode(String str) {
		try {
			return URLEncoder.encode(str, "UTF-8").replace("+", "%20");
		} catch (UnsupportedEncodingException uee) {
			throw new RuntimeException(uee);
		}
	}
}
