/**
 * AngriffsVerteidigungsmittelStaffelung.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.logic.forderungen;

import java.util.ArrayList;
import java.util.Arrays;

import eu.gronos.kostenrechner.controller.TabulierendZeilen;
import eu.gronos.kostenrechner.interfaces.Begruendend;
import eu.gronos.kostenrechner.interfaces.TenorVorbereitend;
import eu.gronos.kostenrechner.logic.TenorUtils;
import eu.gronos.kostenrechner.logic.VollstreckbarkeitsHelfer;
import eu.gronos.kostenrechner.model.forderungen.AufrechnungsForderung;
import eu.gronos.kostenrechner.model.forderungen.Forderung;
import eu.gronos.kostenrechner.model.forderungen.KlageForderung;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;
import eu.gronos.kostenrechner.model.tenordaten.DoubleDataRows;
import eu.gronos.kostenrechner.model.tenordaten.HauptsacheVerhaeltnis;
import eu.gronos.kostenrechner.model.tenordaten.KostenTragungsVerhaeltnis;
import eu.gronos.kostenrechner.model.tenordaten.TenorDatenContainer;
import eu.gronos.kostenrechner.model.tenordaten.VollstreckbarkeitsListe;
import eu.gronos.kostenrechner.model.tenordaten.Vollstreckbarkeit;
import eu.gronos.kostenrechner.model.tenordaten.VollstreckungsVerhaeltnis;
import eu.gronos.kostenrechner.model.tenordaten.TenorDatenContainer.HauptsacheEntscheidungsElemente;
import eu.gronos.kostenrechner.model.tenordaten.TenorDatenContainer.KostenGrundEntscheidungsElemente;
import eu.gronos.kostenrechner.model.tenordaten.TenorDatenContainer.SonstigeEntscheidungsElemente;
import eu.gronos.kostenrechner.model.tenordaten.TenorDatenContainer.StreitwertEntscheidungsElemente;
import eu.gronos.kostenrechner.model.tenordaten.TenorDatenContainer.VollstreckbarkeitsEntscheidungsElemente;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.Allgemein;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.ForderungsStaffelung;

/**
 * Eine Klasse zum Berechnen von Streitwert und von Kostenverteilung bei
 * Staffelung von Klageforderungen und Hilfsaufrechnungen
 * 
 * @author Peter Felix Schuster (SchusterP2)
 * @date 01.08.2014
 */
public class AngriffsVerteidigungsmittelStaffelung implements TenorVorbereitend, Begruendend {
	private final TenorDatenContainer container;

	private static final String[] COLUMN_HEADERS = new String[] { "Angriffs-/Verteidigungsmittel",
			"Höhe der Forderung (EUR)", "Erhöht den Streitwert um (EUR)", "Davon begründet (EUR)",
			"(verbleibende) Klageforderung" };
	private static final String GRUENDE_START_ANGRIFFS_VERTEIDIGUNGS_MITTEL = "Denn das Verhältnis von Obsiegen und Unterliegen bemisst sich anhand folgender Angriffs- und Verteidigungsmittel:\n";
	private static final String BESCHREIBUNG = "Tenor bei Klage/Hilfsaufrechnung (Hauptsachetenor, Kostentenor, Vollstreckbarkeitstenor und Streitwertbeschluss)";
	/**
	 * Die ArrayList<Forderung> mit dem Namen forderungen speichert alle
	 * Forderungen, nach denen zu berechnen ist.
	 */
	private final ArrayList<Forderung> forderungen;
	/**
	 * Der Beteiligter mit dem Namen klaeger speichert Genus/Numerus für die
	 * Tenorerzeugung
	 */
	private final Beteiligter klaeger;
	/**
	 * Der Beteiligter mit dem Namen beklagter speichert Genus/Numerus für die
	 * Tenorerzeugung
	 */
	private final Beteiligter beklagter;
	private double[] hauptsache = null;
	private double[] kosten = null;
	private double streitwert = -1.0;
	private boolean unwesentlichesUnterliegenUebergehen;
	private StringBuilder gruende = new StringBuilder();
	// private ArrayList<Object[]> zeilenListe;
	// private int[] range = new int[2];
	private TabulierendZeilen zeilen;
	private VollstreckbarkeitsListe vollstreckbarkeitsListe;

	/**
	 * Konstruktor:
	 * 
	 * 
	 * @param forderungsStaffelung ein Parameterobjekt {@link ForderungsStaffelung},
	 *                             das beinhaltet:
	 *                             {@link ForderungsStaffelung.forderungen} eine
	 *                             ArrayList<Forderung> mit allen Forderungen, nach
	 *                             denen zu berechnen ist.
	 *                             {@link ForderungsStaffelung#parteien} ein
	 *                             Beteiligter klaeger und ein Beteiligter
	 *                             beklagter, in dem Genus/Numerus für die
	 *                             Tenorerzeugung gespeichert sind
	 * @param allgemein            ein Parameterobjekt {@link Allgemein} mit b92ii =
	 *                             § 92 Abs. 2 ZPO berücksichtigen (d.h.
	 *                             unwesentlichesUnterliegenUebergehen) und ein
	 *                             Unterliegen < 10% wie kein Unterliegen behandeln
	 * @throws IllegalArgumentException Wenn forderungen null oder leer ist; oder
	 *                                  wenn die Reihenfolge (erst Klageforderungen,
	 *                                  dann Aufrechnungsforderungen) nicht
	 *                                  eingehalten ist.
	 * 
	 */
	public AngriffsVerteidigungsmittelStaffelung(Allgemein allgemein, ForderungsStaffelung forderungsStaffelung)
			throws IllegalArgumentException {
		this.forderungen = forderungsStaffelung.forderungen;
		if (forderungen == null || forderungen.size() < 1)
			throw new IllegalArgumentException(TenorUtils.FEHLER_FORDERUNG_LEER);
		if (!isReihenfolgeEingehalten(forderungen))
			throw new IllegalArgumentException(TenorUtils.FEHLER_REIHENFOLGE_FORDERUNGEN);
		setzeForderungenZurueck(forderungen);
		this.klaeger = forderungsStaffelung.parteien.get(0);// klaeger;
		this.beklagter = forderungsStaffelung.parteien.get(1);// .beklagter;
		this.unwesentlichesUnterliegenUebergehen = allgemein.b92ii;// unwesentlichesUnterliegenUebergehen;
		this.container = new TenorDatenContainer();
	}

	/**
	 * Die Methode soll einen Hauptsachetenor erzeugen.
	 * 
	 * @return den Hauptsachetenor als String mit abschließendem Zeilenumbruch
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.TenorVorbereitend#erzeugeHauptsacheEntscheidung()
	 */
	@Override
	public HauptsacheEntscheidungsElemente erzeugeHauptsacheEntscheidung() {
		HauptsacheEntscheidungsElemente entscheidung = new HauptsacheEntscheidungsElemente();
		/* Hauptsache nur berechnen, wenn nicht schon geschehen */
		if (hauptsache == null)
			hauptsache = berechneHauptsache();
		double erfolg = hauptsache[1];
		String tenor;
		if (erfolg > 0) {
			double ganz = hauptsache[0];
			tenor = TenorUtils.satzBeginn(String.format(TenorUtils.VERURTEILUNGS_FORMAT,
					beklagter.parteiBezeichner(Beteiligter.NOMINATIV, beklagter.getLfdNr(), true),
					klaeger.parteiBezeichner(Beteiligter.DATIV, klaeger.getLfdNr(), true), erfolg));
			if (ganz > erfolg) /* wenn nicht vollständig obsiegt oder verloren */
				tenor += TenorUtils.KLAGEABWEISUNG_IM_UEBRIGEN;
		} else
			tenor = TenorUtils.KLAGEABWEISUNG;
		tenor += "\n";
		entscheidung.text = tenor;
		entscheidung.prozessverhaeltnisse.clear();
		entscheidung.prozessverhaeltnisse.add(new HauptsacheVerhaeltnis(klaeger, beklagter, false, (long) erfolg));
		return entscheidung;
	}

	/**
	 * Die Methode soll einen Kostentenor erzeugen.
	 * 
	 * @return einen String
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.TenorVorbereitend#erzeugeKostenEntscheidung()
	 */
	@Override
	public KostenGrundEntscheidungsElemente erzeugeKostenEntscheidung() {
		KostenGrundEntscheidungsElemente kostenEntscheidung = new KostenGrundEntscheidungsElemente();
		kostenEntscheidung.prozessverhaeltnisse.clear();
		/*
		 * Kostenquoten nur berechnen, wenn nicht schon geschehen (vor erster Berechnung
		 * sind sie null).
		 */
		if (kosten == null)
			kosten = berechneKostenverteilung();
		if ((kosten[0] <= 0 && !unwesentlichesUnterliegenUebergehen)
				|| (kosten[0] < 0.1 && unwesentlichesUnterliegenUebergehen)) {
			kostenEntscheidung.text = String.format(TenorUtils.KOSTENTRAGUNG_EINSEITIG,
					beklagter.parteiBezeichner(Beteiligter.NOMINATIV, beklagter.getLfdNr(), true));
			kostenEntscheidung.prozessverhaeltnisse.add(new KostenTragungsVerhaeltnis(klaeger, beklagter, false, 1.0));
		} else if ((kosten[1] <= 0 && !unwesentlichesUnterliegenUebergehen)
				|| (kosten[1] < 0.1 && unwesentlichesUnterliegenUebergehen)) {
			kostenEntscheidung.text = String.format(TenorUtils.KOSTENTRAGUNG_EINSEITIG,
					klaeger.parteiBezeichner(Beteiligter.NOMINATIV, klaeger.getLfdNr(), true));
			kostenEntscheidung.prozessverhaeltnisse.add(new KostenTragungsVerhaeltnis(beklagter, klaeger, false, 1.0));
		} else {
			kostenEntscheidung.text = String.format(TenorUtils.KOSTENVERTEILUNG_FORMAT,
					klaeger.parteiBezeichner(Beteiligter.NOMINATIV, klaeger.getLfdNr(), true), kosten[0] * 100,
					beklagter.parteiBezeichner(Beteiligter.NOMINATIV, beklagter.getLfdNr(), true), kosten[1] * 100);
			kostenEntscheidung.prozessverhaeltnisse
					.add(new KostenTragungsVerhaeltnis(beklagter, klaeger, false, kosten[0]));
			kostenEntscheidung.prozessverhaeltnisse
					.add(new KostenTragungsVerhaeltnis(klaeger, beklagter, false, kosten[1]));
		}
		if (kosten[0] <= 0 || kosten[1] <= 0)
			starteGruende(TenorUtils.KOSTENENTSCHEIDUNG_91);
		else if (unwesentlichesUnterliegenUebergehen && (kosten[0] <= 0.1 || kosten[1] <= 0.1))
			starteGruende(TenorUtils.KOSTENENTSCHEIDUNG_91_92_II);
		else
			starteGruende(TenorUtils.KOSTENENTSCHEIDUNG_92_II);
		erweitereGruende(GRUENDE_START_ANGRIFFS_VERTEIDIGUNGS_MITTEL);
		zeilen.toStringBuilder(gruende);
		return kostenEntscheidung;
	}

	/**
	 * Die Methode erzeugt den Tenor zur vorläufigen Vollstreckbarkeit
	 * 
	 * @return einen String
	 */
	@Override
	public VollstreckbarkeitsEntscheidungsElemente erzeugeVollstreckbarkeitsEntscheidung() {
		VollstreckbarkeitsEntscheidungsElemente vollEntscheidung = new VollstreckbarkeitsEntscheidungsElemente();
		String schuldner = TenorUtils.JEWEILIGER_VOLLSTRECKUNGSSCHULDNER;
		String glaeubiger = TenorUtils.JEWEILIGER_VOLLSTRECKUNGSGLAEUBIGER;
		if (vollstreckbarkeitsListe == null)
			vollstreckbarkeitsListe = berechneVollstreckbarkeit();
		erweitereGruende(vollstreckbarkeitsListe.toString());
		VollstreckbarkeitsHelfer helfer = new VollstreckbarkeitsHelfer(
				container.hauptsacheEntscheidung.prozessverhaeltnisse,
				container.kostenEntscheidung.prozessverhaeltnisse);
		vollEntscheidung.prozessverhaeltnisse = helfer.toVollstreckbarkeitsVerhaeltnisse(klaeger, beklagter,
				vollstreckbarkeitsListe);
		if (vollstreckbarkeitsListe.sindAlle708iVm713()) {
			vollEntscheidung.text = TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR;
			return vollEntscheidung;
		} else if (vollstreckbarkeitsListe.sindAlle709()) {
			vollEntscheidung.text = TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_709;
			return vollEntscheidung;
		} else if (vollstreckbarkeitsListe.sindAlle708iVm711()) {
			/*
			 * Parteien einsetzen: Wenn die Liste der Vollstreckungsverhaeltnisse nur einen
			 * Eintrag hat, darf nur eine Partei vollstrecken. Da hilft die Liste der
			 * VollstreckungsVerhältnisse.
			 */
			if (1 == vollEntscheidung.prozessverhaeltnisse.size()) {
				VollstreckungsVerhaeltnis verhaeltnis = vollEntscheidung.prozessverhaeltnisse.get(0);
				schuldner = verhaeltnis.getSchuldner().parteiBezeichner(Beteiligter.DATIV,
						verhaeltnis.getSchuldner().getLfdNr(), true);
				glaeubiger = verhaeltnis.getGlaeubiger().parteiBezeichner(Beteiligter.NOMINATIV,
						verhaeltnis.getGlaeubiger().getLfdNr(), true);
			}
			/*
			 * Ansonsten dürfen beide vollstrecken (und es bleibt beim
			 * "jeweiligen Vollstreckungsschuldner-/gläubiger".
			 */
			vollEntscheidung.text = TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR + TenorUtils.satzBeginn(
					String.format(TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_708_711, schuldner, glaeubiger));
			return vollEntscheidung;
		}
		/*
		 * Gemischter Vollstreckbarkeitstenor: die VollstreckungsVerhaeltnisse
		 * durchgehen. Dafür haben wir sie ja.
		 */
		StringBuffer tenor = new StringBuffer(TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR);
		for (VollstreckungsVerhaeltnis verhaeltnis : vollEntscheidung.prozessverhaeltnisse) {
			glaeubiger = verhaeltnis.getGlaeubiger().parteiBezeichner(Beteiligter.NOMINATIV,
					verhaeltnis.getGlaeubiger().getLfdNr(), true);
			schuldner = verhaeltnis.getSchuldner().parteiBezeichner(Beteiligter.DATIV,
					verhaeltnis.getSchuldner().getLfdNr(), true);
			if (verhaeltnis.getVollstreckbarkeit().isB709s1()) {
				tenor.append(String.format(TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_709_KOMBI, schuldner));
			} else {
				tenor.append(TenorUtils.satzBeginn(String.format(TenorUtils.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_708_711,
						TenorUtils.satzBeginn(schuldner), glaeubiger)));
			}
		}
		vollEntscheidung.text = tenor.toString();
		return vollEntscheidung;
	}

	/**
	 * Die Methode soll eine Streitwertfestsetzung erzeugen.
	 * 
	 * @return
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.TenorVorbereitend#erzeugeStreitwertEntscheidung()
	 */
	@Override
	public StreitwertEntscheidungsElemente erzeugeStreitwertEntscheidung() {
		// String festsetzung;
		StreitwertEntscheidungsElemente swe = new StreitwertEntscheidungsElemente();
		/*
		 * Streitwert nur berechnen, wenn nicht schon geschehen (vor erster Berechnung
		 * ist er < 0).
		 */
		if (streitwert < 0)
			streitwert = berechneStreitwert();
		swe.streitwerte.add(new Double(streitwert).longValue());
		swe.text = String.format(TenorUtils.STREITWERT_FESTGESETZT_EINFACH, streitwert);
		swe.text += "\n";
		return swe;
	}

	/**
	 * Die Methode erzeugt einen leeren sonstigen Tenor.
	 * 
	 * @return ein {@link SonstigeEntscheidungsElemente}
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.TenorVorbereitend#erzeugeSonstigeEntscheidung()
	 */
	@Override
	public SonstigeEntscheidungsElemente erzeugeSonstigeEntscheidung() {
		SonstigeEntscheidungsElemente sonstige = new SonstigeEntscheidungsElemente();
		sonstige.text = "";
		return sonstige;
	}

	/**
	 * Die Methode gibt die Begründung für den Kostentenor als String zurück.
	 * 
	 * @return den Text Gründe/Entscheidungsgründe als String
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.Begruendend#getGruende()
	 */
	@Override
	public String getGruende() {
		return this.gruende.toString();
	}

	/**
	 * Die Methode dient dazu, den gesamten {@link TenorDatenContainer} zu erzeugen.
	 * 
	 * @return das Ergebnis von erzeugeHauptsachetenor() + erzeugeKostentenor() +
	 *         erzeugeStreitwertFestsetzung()
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.TenorVorbereitend#erzeugeContainer()
	 */
	@Override
	public TenorDatenContainer erzeugeContainer() {
		container.berechnungsTyp = getBerechnungsTyp();
		container.hauptsacheEntscheidung = erzeugeHauptsacheEntscheidung();
		container.kostenEntscheidung = erzeugeKostenEntscheidung();
		container.vollstreckbarkeitsEntscheidung = erzeugeVollstreckbarkeitsEntscheidung();
		container.streitwertEntscheidung/* .text */ = erzeugeStreitwertEntscheidung();
		container.sonstigeEntscheidung = erzeugeSonstigeEntscheidung();
		container.begruendung = zeilen.toBegruendungsElemente(getGruende());
		return container;
	}

	/**
	 * Die Methode soll eine Beschreibung für die Art des Tenor zurückgeben, damit
	 * andere Klassen etwa das Dokument danach benennen können.
	 * 
	 * @return
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.TenorVorbereitend#getBerechnungsTyp()
	 */
	@Override
	public String getBerechnungsTyp() {
		return BESCHREIBUNG;
	}

	/**
	 * Die Methode dient dazu, die Reihenfolge der Forderungen zu prüfen!
	 * 
	 * @param forderungen die zu prüfende ArrayList<Forderung>
	 * @return true, wenn die Reihenfolge eingehalten ist: Erste muss Klageforderung
	 *         sein, dann alle übrigen Klageforderungen, dann Aufrechnungen (und
	 *         keine Klageforderungen mehr)
	 */
	private boolean isReihenfolgeEingehalten(ArrayList<Forderung> forderungen) {
		boolean antwort = true;
		boolean umstiegAufAufrechnung = false;
		if (!(forderungen.get(0) instanceof KlageForderung))
			antwort = false;
		for (Forderung forderung : forderungen) {
			if (forderung instanceof AufrechnungsForderung)
				umstiegAufAufrechnung = true;
			if (umstiegAufAufrechnung && forderung instanceof KlageForderung)
				antwort = false;
		}
		return antwort;
	}

	/**
	 * Die Methode dient dazu, bei allen {@link AufrechnungsForderung
	 * AufrechnungsForderungen} der <code>liste</code>
	 * {@link AufrechnungsForderung#setzeEffektivenErfolgZurueck()} und
	 * {@link AufrechnungsForderung#setzeEffektivenWertZurueck()} durchzuführen. Das
	 * muss bei Initialiiserung der Berechnung durchgeführt werden. Wenn seit der
	 * letzten Berechnung Werte verändert wurden, kommt es sonst zu Fehlern.
	 * 
	 * @param liste eine {@link ArrayList}&lt;{@link Forderung}&gt;
	 */
	private void setzeForderungenZurueck(ArrayList<Forderung> liste) {
		for (Forderung forderung : liste)
			if (forderung instanceof AufrechnungsForderung) {
				AufrechnungsForderung aufrechnung = (AufrechnungsForderung) forderung;
				aufrechnung.setzeEffektivenErfolgZurueck();
				aufrechnung.setzeEffektivenWertZurueck();
			}
		;
	}

	/**
	 * Die Methode berechnet das Obsiegen des Klägers in der Hauptsache und
	 * ermittelt dabei auch die vollständige Klageforderung.
	 * 
	 * @return ein Array double[], bei dem index 0 die komplette Klage ist und index
	 *         1 die verbleibende Klageforderung für den Tenor.
	 */
	private double[] berechneHauptsache() {
		double verbleibend = 0;
		double volleForderung = 0;
		for (Forderung f : forderungen) {
			if (f instanceof KlageForderung) {
				/* Das Obsiegen bzgl. Klageforderungen wird zusammengerechnet */
				verbleibend += f.getErfolg();
				/* Auch der Wert, sofern nicht wirtschaftlich identisch */
				if (!((KlageForderung) f).isWirtschaftlichIdentisch())
					volleForderung += f.getWert();
			} else if (f instanceof AufrechnungsForderung) {
				/* Aufrechnungen werden abgezogen, soweit noch etwas übrig ist. */
				AufrechnungsForderung af = (AufrechnungsForderung) f;
				/*
				 * Dabei wird abgefragt, ob der effektive Erfolg schon berechnet wurde (dann ist
				 * getEffektiverErfolg nicht mehr < 0)
				 */
				verbleibend -= af.getEffektiverErfolg() < 0 ? af.berechneEffektivenErfolg(verbleibend)
						: af.getEffektiverErfolg();
			}
		}
		return new double[] { volleForderung, verbleibend };
	}

	/**
	 * Die Methode berechnet die Kostenverteilung, damit das Ergebnis von
	 * {@link AngriffsVerteidigungsmittelStaffelung#erzeugeKostenEntscheidung()
	 * erzeugeKostentenor()} ausgelesen werden kann.
	 * 
	 * @return ein Array double[] mit der Quote des Klägers und des Beklagten,
	 *         jeweils zwischen 0.0 und 1.0
	 * 
	 * @see AngriffsVerteidigungsmittelStaffelung#erzeugeKostenEntscheidung()
	 */
	private double[] berechneKostenverteilung() {
		/*
		 * Streitwert nur berechnen, wenn nicht schon geschehen (vor erster Berechnung
		 * ist er < 0).
		 */
		if (streitwert < 0)
			streitwert = berechneStreitwert();
		double klaegerunterliegen = 0.0;
		double beklagtenunterliegen = 0.0;
		double verbleibend = 0.0;
		int aufrechnungen = 0;
		/*
		 * Liste der Zeilen der Begründungstabelle anlegen und mit der Kopfzeile
		 * beginnen
		 */
		zeilen = new TabulierendZeilen();
		zeilen.add(new ArrayList<String>(Arrays.asList(COLUMN_HEADERS)));
		for (Forderung f : forderungen) {
			DoubleDataRows doubles = new DoubleDataRows(4);
			String beschreibung;
			if (f instanceof KlageForderung) {
				beschreibung = f.getBeschreibung();
				/* Das Obsiegen bzgl. Klageforderungen wird zusammengerechnet */
				verbleibend += f.getErfolg();
				/*
				 * Das Unterliegen bemisst sich hier allein nach Wert und Erfolg.
				 */
				klaegerunterliegen += f.getWert() - f.getErfolg();
				beklagtenunterliegen += f.getErfolg();
				/*
				 * Bei den Hauptklageforderungen muss einiges doppelt in die Begründungstabelle
				 * ausgegeben werden.
				 */
				doubles.add(new Double(f.getWert()));
				doubles.add(new Double(f.getWert()));
				doubles.add(new Double(f.getErfolg()));
				doubles.add(new Double(verbleibend));
				zeilen.add(beschreibung, doubles);
			} else if (f instanceof AufrechnungsForderung) {
				aufrechnungen++;
				beschreibung = String.format("%d. %s", aufrechnungen, f.getBeschreibung());
				/* Aufrechnungen werden abgezogen, soweit noch etwas übrig ist. */
				AufrechnungsForderung af = (AufrechnungsForderung) f;
				/*
				 * Das Unterliegen bemisst sich hier nach effektivem Wert und effektivem Erfolg.
				 * Dabei wird abgefragt, ob der effektive Erfolg schon berechnet wurde (dann ist
				 * getEffektiverErfolg nicht mehr < 0)
				 */
				double effektiverErfolg = af.getEffektiverErfolg() < 0 ? af.berechneEffektivenErfolg(verbleibend)
						: af.getEffektiverErfolg();
				double effektiverWert = af.getEffektiverWert() < 0 ? af.berechneEffektivenWert(verbleibend)
						: af.getEffektiverWert();
				verbleibend -= effektiverErfolg;
				klaegerunterliegen += effektiverErfolg;
				beklagtenunterliegen += effektiverWert - effektiverErfolg;
				/*
				 * Bei den Aufrechnungen wird alles in der Begründungstabelle gebraucht
				 */
				doubles.add(new Double(f.getWert()));
				doubles.add(new Double(af.getEffektiverWert()));
				doubles.add(new Double(af.getEffektiverErfolg()));
				doubles.add(new Double(verbleibend));
				zeilen.add(beschreibung, doubles);
			}
		}
		/* Eine Ergebniszeile gibt es hier nicht */
		return new double[] { klaegerunterliegen / streitwert, beklagtenunterliegen / streitwert };
	}

	/**
	 * Die Methode berechnet, inwieweit das Urteil vorläufig vollstreckbar ist.
	 * 
	 * @return eine {@link VollstreckbarkeitsListe}, deren 0. Element die
	 *         Unanfechtbarkeit speichert, wenn nicht unanfechtbar das 1. Element
	 *         das Verhältnis kl -> bk und das 2. Element das Verhältnis bk -> kl.
	 * 
	 */
	private VollstreckbarkeitsListe berechneVollstreckbarkeit() {
		VollstreckbarkeitsListe vollstreckbarkeitsListe = new VollstreckbarkeitsListe();
		Vollstreckbarkeit unanfechtbarkeit = pruefeUnanfechtbarkeit();
		vollstreckbarkeitsListe.add(unanfechtbarkeit);
		if (unanfechtbarkeit.isB713())
			return vollstreckbarkeitsListe;
		/*
		 * Kostenquoten nur berechnen, wenn nicht schon geschehen (vor erster Berechnung
		 * sind sie null).
		 */
		if (kosten == null)
			kosten = berechneKostenverteilung();
		/*
		 * Beim Kläger hinsichtlich Hauptsache einsetzen, was von seiner
		 * Hauptklageforderung begründet ist (hauptsache[1]) und hinsichtlich der Kosten
		 * die üblichen Klägerkosten, soweit sie der Beklagte tragen muss (kosten[1])
		 */
		vollstreckbarkeitsListe.add(Vollstreckbarkeit.pruefeSicherheitsleistung((long) hauptsache[1],
				(long) (TenorUtils.errechneGebuehrenSumme(TenorUtils.KOSTEN_KLAEGER, (long) streitwert) * kosten[1])));
		/*
		 * Beim Kläger hinsichtlich Hauptsache 0 einsetzen und hinsichtlich der Kosten
		 * die üblichen Klägerkosten, soweit sie der Kläger tragen muss (kosten[0])
		 */
		vollstreckbarkeitsListe.add(Vollstreckbarkeit.pruefeSicherheitsleistung(0L,
				(long) (TenorUtils.errechneGebuehrenSumme(TenorUtils.KOSTEN_BEKLAGTER, (long) streitwert)
						* kosten[0])));
		return vollstreckbarkeitsListe;
	}

	/**
	 * Die Methode prüft, ob das Urteil unanfechtbar ist (§§ 708, 713 ZPO).
	 * 
	 * @return ein {@link Vollstreckbarkeit}, das im Fall der
	 *         Unanfechtbarkeit 708 und 713 auf <code>true</code> setzt. Ansonsten
	 *         enthaelt es nichts.
	 */
	private Vollstreckbarkeit pruefeUnanfechtbarkeit() {
		Vollstreckbarkeit unanfechtbarkeit = new Vollstreckbarkeit();
		/*
		 * Streitwert nur berechnen, wenn nicht schon geschehen (vor erster Berechnung
		 * ist er < 0).
		 */
		if (streitwert < 0)
			streitwert = berechneStreitwert();
		if (streitwert <= TenorUtils.BERUFUNGS_GRENZE) {
			unanfechtbarkeit = new Vollstreckbarkeit(true, false, false, false, true);
		} else if (streitwert <= TenorUtils.BERUFUNGS_GRENZE * 2) {
			/* Hauptsache nur berechnen, wenn nicht schon geschehen */
			if (hauptsache == null)
				hauptsache = berechneHauptsache();
			/*
			 * Wie ist der Kläger beschwert? Das, was von seiner Klage nicht durchgedrungen
			 * ist.
			 */
			double klaeger = hauptsache[0] - hauptsache[1];
			/* Wie ist der Beklagte beschwert: Das wozu er verurteilt wurde ... */
			double beklagter = hauptsache[1];
			/*
			 * ... und das, was er an Aufrechnungsforderungen "verbraten" hat, ob mit oder
			 * ohne Erfolg.
			 */
			for (Forderung f : forderungen) {
				if (f instanceof AufrechnungsForderung)
					beklagter += ((AufrechnungsForderung) f).getEffektiverWert();
			}
			if (klaeger <= TenorUtils.BERUFUNGS_GRENZE && beklagter <= TenorUtils.BERUFUNGS_GRENZE) {
				unanfechtbarkeit = new Vollstreckbarkeit(true, false, false, false, true);
			}
		}
		return unanfechtbarkeit;
	}

	/**
	 * Die Methode berechnet den Streitwert.
	 * 
	 * @return den Streitwert als double
	 */
	private double berechneStreitwert() {
		double wert = 0.0;
		double verbleibend = 0.0;
		for (Forderung f : forderungen) {
			if (f instanceof KlageForderung) {
				/* Das Obsiegen bzgl. Klageforderungen wird zusammengerechnet */
				verbleibend += f.getErfolg();
				/* Auch der Wert, sofern nicht wirtschaftlich identisch */
				if (!((KlageForderung) f).isWirtschaftlichIdentisch())
					wert += f.getWert();
			} else if (f instanceof AufrechnungsForderung) {
				/* Aufrechnungen werden abgezogen, soweit noch etwas übrig ist. */
				AufrechnungsForderung af = (AufrechnungsForderung) f;
				/*
				 * Dabei wird abgefragt, ob der effektive Wert und der effektive Erfolg schon
				 * berechnet wurden (dann sind getEffektiverWert bzw. getEffektiverErfolg nicht
				 * mehr < 0)
				 */
				wert += af.getEffektiverWert() < 0 ? af.berechneEffektivenWert(verbleibend) : af.getEffektiverWert();
				verbleibend -= af.getEffektiverErfolg() < 0 ? af.berechneEffektivenErfolg(verbleibend)
						: af.getEffektiverErfolg();
			}
		}
		return wert;
	}

	/**
	 * Die Methode setzt die Entscheidungsgründe zurück (sofern nicht leer) und
	 * beginnt sie neu mit dem übergebenen Text.
	 * 
	 * @param text eine {@link CharSequence} mit dem Anfangssatz(-bestandteil) der
	 *             Gründe.
	 */
	private void starteGruende(CharSequence text) {
		if (gruende.length() > 0)
			gruende.delete(0, gruende.length());
		erweitereGruende(text);
	}

	/**
	 * Die Methode erweitert die bisherigen Gründe
	 * 
	 * @param text eine {@link CharSequence} mit den weiteren Gründen.
	 */
	private void erweitereGruende(CharSequence text) {
		gruende.append(text);
	}

}
