/**
 * AnwaltsGebuehrenTabelle.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.model.gebuehren;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Calendar;

import javax.xml.bind.JAXB;

/**
 * Das ist eine Klasse, um Gebührenhöhe und -sätze nach dem RVG zu berechnen und
 * zu speichern (also Rechtsanwaltsgebühren).
 * 
 * § 13 RVG Wertgebühren (1) Wenn sich die Gebühren nach dem Gegenstandswert
 * richten, beträgt die Gebühr bei einem Gegenstandswert bis 500 Euro 45 Euro.
 * Die Gebühr erhöht sich bei einem
 * 
 * Gegenstandswert bis ... Euro, für jeden angefangenen Betrag von weiteren ...
 * Euro, um ... Euro
 * 
 * 2000, 500, 35
 * 
 * 10000, 1000, 51
 * 
 * 25000, 3000, 46
 * 
 * 50000, 5000, 75
 * 
 * 200000, 15000, 85
 * 
 * 500000, 30000, 120
 * 
 * über 500000, 50000, 150
 * 
 * (2) Der Mindestbetrag einer Gebühr ist 15 Euro.
 * 
 * @author Peter Schuster (setrok)
 * @date 12.01.2014
 * 
 */
public class AnwaltsGebuehrenTabelle extends GebuehrenTabelle {

	public static final GebuehrenTatbestand VV1003 = new GebuehrenSatzTatbestand(
			"Einigungsgebühr (im Gerichtsverfahren), Ziff. 1003 VV RVG", 1.0, AnwaltsGebuehrenTabelle.class);

	public static final GebuehrenTatbestand VV1008 = new GebuehrenErhoehungsTatbestand(
			"Erhöhung der Geschäfts- oder Verfahrensgebühr, Ziffer 1008 VV RVG", AnwaltsGebuehrenTabelle.class, 0.3,
			2.0);

	public static final GebuehrenSatzTatbestand VV3100 = new GebuehrenSatzTatbestand(
			"Verfahrensgebühr, Ziff. 3100 VV RVG", 1.3, AnwaltsGebuehrenTabelle.class);

	public static final GebuehrenTatbestand VV3101 = new GebuehrenSatzTatbestand(
			"Verfahrensgebühr (ermäßigt), Ziff. 3101 VV RVG", 0.8, AnwaltsGebuehrenTabelle.class);

	public static final GebuehrenSatzTatbestand VV3104 = new GebuehrenSatzTatbestand("Terminsgebühr, Ziff. 3104 VV RVG",
			1.2, AnwaltsGebuehrenTabelle.class);

	public static final GebuehrenSatzTatbestand VV3105 = new GebuehrenSatzTatbestand(
			"Terminsgebühr (ermäßigt), Ziff. 3105 VV RVG", 0.5, AnwaltsGebuehrenTabelle.class);

	public static final GebuehrenSatzTatbestand VV3200 = new GebuehrenSatzTatbestand(
			"Verfahrensgebühr Rechtsmittel, Ziff. 3200 VV RVG", 1.6, AnwaltsGebuehrenTabelle.class);

	public static final AuslagenTatbestand VV7002 = new AuslagenTatbestand(
			"Auslagenpauschale für Entgelte für Post- und Telekommunikationsdienstleistungen, Ziff. 7002 VV RVG", 20.0,
			AnwaltsGebuehrenTabelle.class);

	public static final GebuehrenTatbestand VV7008 = new MehrwertsteuerTatbestand(
			"Umsatzsteuer auf die Vergütung, Ziff. 7008 VV RVG", AnwaltsGebuehrenTabelle.class, 0.19);

	public static final GebuehrenTatbestand VV7008_R = new MehrwertsteuerTatbestand(
			"Umsatzsteuer auf die Vergütung, Ziff. 7008 VV RVG (mit anschließendem Reset)",
			AnwaltsGebuehrenTabelle.class, 0.19, true);

	public static final GebuehrenTatbestand VV7008_0 = new MehrwertsteuerTatbestand(
			"Keine Umsatzsteuer auf die Vergütung, Ziff. 7008 VV RVG (bei Befreiung)", AnwaltsGebuehrenTabelle.class,
			0.0, true);

	public static final String GESETZ = "Rechtsanwaltsvergütungsgesetz (RVG)";

	/**
	 * Der Konstruktor ruft den Konstruktor der Oberklasse auf. Ein
	 * Gültigkeitszeitraum wird nicht festgelegt. Zudem initialisiert er mittels
	 * {@link eu.gronos.kostenrechner.model.gebuehren.AnwaltsGebuehrenTabelle#fuelleTabelle()}
	 * die Gebührentabelle.
	 * 
	 * @see eu.gronos.kostenrechner.model.gebuehren.GebuehrenTabelle#GebuehrenTabelle(long,long,Calendar,Calendar)
	 */
	public AnwaltsGebuehrenTabelle() {
		super();
		InputStream stream = getClass().getClassLoader().getResourceAsStream("resources/rvg.xml");
		setTabelle(readTabelleFromStream(stream));
	}

	/**
	 * Der Konstruktor ruft den Konstruktor der Oberklasse auf. Zudem initialisiert
	 * er mittels {@link GebuehrenTabelle#readTabelleFromStream(InputStream)} und
	 * {@link #setTabelle(ArrayList)} mittels XML-Datei und {@link JAXB} die Werte
	 * der Tabelle.
	 * 
	 * @param stream ein {@link InputStream} von einer XML-Datei für {@link JAXB}
	 */
	public AnwaltsGebuehrenTabelle(InputStream stream) {
		super();
		setTabelle(readTabelleFromStream(stream));
	}

	/**
	 * Die Methode getAllGebuehrenTatbestaende dient dazu, alle vorbereiteten
	 * Instanzen von GebuehrenTatbestand, die zu der GebuehrenTabelle gehören, zu
	 * bekommen.
	 * 
	 * @return ein Array GebuehrenTatbestand[] mit den Tatbeständen.
	 */
	public static GebuehrenTatbestand[] getAllGebuehrenTatbestaende() {
		return new GebuehrenTatbestand[] { VV1003, VV1008, VV3100, VV3101, VV3104, VV3105, VV3200, VV7002, VV7008,
				VV7008_R, VV7008_0 };
	}
}
