/**
 * AbstractGebuehrenTatbestand.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.model.gebuehren;

import java.io.Serializable;

import javax.xml.bind.JAXB;
import javax.xml.bind.annotation.XmlAttribute;

/**
 * Eine Oberklasse, um GebührenTatbestände zu speichern, entweder aus dem VV RVG
 * oder dem KV GKG. Als Oberklasse für alle Gebührentatbestände nur mit
 * beschreibung und Verweis auf die Klasse der Gebührenordnung. Abgeleitete
 * Klassen können den eigentlichen Inhalt wie Gebührensatz, Auslagenhöhe oder
 * Mehrwertsteuersatz bereitstellen.
 * 
 * @author Peter Felix Schuster (SchusterP2)
 * @date 21.08.2014
 */
public abstract class GebuehrenTatbestand implements Serializable {

	private static final long serialVersionUID = -7148753363484353653L;
	private String bezeichnung;
	private Class<? extends GebuehrenTabelle> gebuehrenKlasse;

	/**
	 * Erstellt eine Instanz der Klasse mit den angegebenen Werten.
	 * 
	 * @param bezeichnung     Die Bezeichnung des Gebührentatbestands nach dem
	 *                        jeweiligen Verzeichnis als String, z.B. "Nr. 1100 KV"
	 * @param gebuehrenKlasse Klasse der Gebühr, also den Verweis auf die Klasse
	 *                        eines Ablegers von GebuehrenTabelle, also entweder
	 *                        GerichtsGebuehrenTabelle.class oder
	 *                        AnwaltsGebuehrenTabelle.class
	 * @throws ClassNotFoundException
	 */
	public GebuehrenTatbestand(String bezeichnung, Class<? extends GebuehrenTabelle> gebuehrenKlasse) {
		super();
		this.bezeichnung = bezeichnung;
		this.gebuehrenKlasse = gebuehrenKlasse;
	}

	/**
	 * Konstruktor ohne Parameter für {@link JAXB}
	 * 
	 */
	public GebuehrenTatbestand() {
		super();
	}

	/**
	 * Die Methode gibt die Bezeichnung des Tatbestands als String zurück. Dieser
	 * sollte den jeweiligen Tatbestand einigermaßen eindeutig beschreiben.
	 * 
	 * @return Die Bezeichnung des Gebührentatbestands nach dem jeweiligen
	 *         Verzeichnis als String, z.B. "Nr. 1100 KV"
	 */
	@XmlAttribute(name = "bezeichnung")
	public String getBezeichnung() {
		return bezeichnung;
	}

	/**
	 * @param bezeichnung d. {@link #bezeichnung}, d. gesetzt werden soll als String
	 */
	public void setBezeichnung(String bezeichnung) {
		this.bezeichnung = bezeichnung;
	}

	/**
	 * @return gebuehrenKlasse gibt zurück, auf welche Gebührentabelle sich der
	 *         Tatbestand bezieht, also eine Klasse, die GebuehrenTabelle erweitert.
	 */
	@XmlAttribute(name = "gesetz")
	public Class<? extends GebuehrenTabelle> getGebuehrenKlasse() {
		return gebuehrenKlasse;
	}

	/**
	 * @param gebuehrenKlasse d. {@link #gebuehrenKlasse}, d. gesetzt werden soll
	 *                        als Class<? extends GebuehrenRechner>
	 */
	public void setGebuehrenKlasse(Class<? extends GebuehrenTabelle> gebuehrenKlasse) {
		this.gebuehrenKlasse = gebuehrenKlasse;
		/* Class reparieren, wenn null, weil XML von alter Version */
		if (this.gebuehrenKlasse == null && bezeichnung != null) {
			if (bezeichnung.contains("VV RVG")) {
				this.gebuehrenKlasse = AnwaltsGebuehrenTabelle.class;
			} else if (bezeichnung.contains("KV GKG")) {
				this.gebuehrenKlasse = GerichtsGebuehrenTabelle.class;
			}
		}
	}

	/**
	 * Die Methode dient dazu, einen {@link Object#hashCode()} zu erstellen.
	 * 
	 * @return
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((bezeichnung == null) ? 0 : bezeichnung.hashCode());
		result = prime * result + ((gebuehrenKlasse == null) ? 0 : gebuehrenKlasse.hashCode());
		return result;
	}

	/**
	 * Vergleicht nach den Kriterien {@link #getBezeichnung()} und
	 * {@link #getGebuehrenKlasse()}
	 * 
	 * @param obj das zu vergleichende {@link Object}
	 * @return
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof GebuehrenTatbestand)) {
			return false;
		}
		GebuehrenTatbestand other = (GebuehrenTatbestand) obj;
		if (bezeichnung == null) {
			if (other.bezeichnung != null) {
				return false;
			}
		} else if (!bezeichnung.equals(other.bezeichnung)) {
			return false;
		}
		if (gebuehrenKlasse == null) {
			if (other.gebuehrenKlasse != null) {
				return false;
			}
		} else if (!gebuehrenKlasse.equals(other.gebuehrenKlasse)) {
			return false;
		}
		return true;
	}

}