/**
 * GerichtsGebuehrenTabelle.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.model.gebuehren;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Calendar;

import javax.xml.bind.JAXB;

/**
 * Das ist eine Klasse, um Gebührenhöhe und -sätze nach dem GKG zu berechnen und
 * zu speichern (also Gerichtsgebühren).
 * 
 * § 34 GKG Wertgebühren (1) Wenn sich die Gebühren nach dem Streitwert richten,
 * beträgt die Gebühr bei einem Streitwert bis 500 Euro 35 Euro. Die Gebühr
 * erhöht sich bei einem
 * 
 * Streitwert bis ... Euro, für jeden angefangenen Betrag von weiteren ... Euro,
 * um ... Euro
 * 
 * 2000, 500, 18
 * 
 * 10000, 1000, 19
 * 
 * 25000, 3000, 26
 * 
 * 50000, 5000, 35
 * 
 * 200000, 15000, 120
 * 
 * 500000, 30000, 179
 * 
 * über 500000, 50000, 180 .
 * 
 * (2) Der Mindestbetrag einer Gebühr ist 15 Euro.
 * 
 * 
 * @author Peter Schuster (setrok)
 * @date 12.01.2014
 * 
 */
public class GerichtsGebuehrenTabelle extends GebuehrenTabelle {

	/**
	 * Der double mit dem Namen KV_1100 speichert die Gebühr Nr. 1110 KV-GKG.
	 * Verfahren über den Antrag auf Erlass eines Mahnbescheids oder eines
	 * Europäischen Zahlungsbefehls.
	 */
	static final GebuehrenSatzTatbestand KV1100 = new GebuehrenSatzTatbestand(
			"Verfahren auf Erlass eines Mahnbescheids, Ziff. 1100 KV GKG", 0.5,
			GerichtsGebuehrenTabelle.class);
	/**
	 * Der double mit dem Namen KV_1210 speichert die Gebühr für Verfahren im
	 * allgemeinen vor Amts- und Landgerichten (Nr. 1210 KV-GKG). (1) Soweit
	 * wegen desselben Streitgegenstands ein Mahnverfahren vorausgegangen ist,
	 * entsteht die Gebühr mit dem Eingang der Akten bei dem Gericht, an das der
	 * Rechtsstreit nach Erhebung des Widerspruchs oder Einlegung des Einspruchs
	 * abgegeben wird; in diesem Fall wird eine Gebühr 1100 nach dem Wert des
	 * Streitgegenstands angerechnet, der in das Prozessverfahren übergegangen
	 * ist. Satz 1 gilt entsprechend, wenn wegen desselben Streitgegenstands ein
	 * Europäisches Mahnverfahren vorausgegangen ist. (2) Soweit der Kläger
	 * wegen desselben Streitgegenstands einen Anspruch zum Musterverfahren
	 * angemeldet hat (§ 10 Abs. 2 KapMuG), wird insoweit die Gebühr 1902
	 * angerechnet.
	 */
	public static final GebuehrenSatzTatbestand KV1210 = new GebuehrenSatzTatbestand(
			"Verfahren im Allgemeinen (erster Rechtszug), Ziff. 1210 KV GKG",
			3.0, GerichtsGebuehrenTabelle.class);

	/**
	 * Der double mit dem Namen KV_1211 speichert die ermäßigte Gebühr Nr. 1211
	 * KV-GKG.
	 */
	public static final GebuehrenSatzTatbestand KV1211 = new GebuehrenSatzTatbestand(
			"Verfahren im Allgemeinen (erster Rechtszug, ermäßigt), Ziff. 1211 KV GKG",
			1.0, GerichtsGebuehrenTabelle.class);
	/**
	 * Der double mit dem Namen KV_1212 speichert die Verfahrensgebühr für
	 * erstinstanzliche Verfahren vor dem OLG
	 */
	public static final GebuehrenSatzTatbestand KV1212 = new GebuehrenSatzTatbestand(
			"Verfahren vor dem OLG (1. Instanz), Ziff. 1212 KV GKG", 4.0,
			GerichtsGebuehrenTabelle.class);

	/**
	 * KV1213 speichert Verfahren vor dem OLG (1. Instanz, ermäßigt), Ziff. 1213
	 * KV GKG als GebuehrenSatzTatbestand.
	 */
	public static final GebuehrenSatzTatbestand KV1213 = new GebuehrenSatzTatbestand(
			"Verfahren vor dem OLG (1. Instanz, ermäßigt), Ziff. 1213 KV GKG",
			2.0, GerichtsGebuehrenTabelle.class);
	/**
	 * Der double mit dem Namen KV_1220 speichert Nr. 1220 KV-GKG. Berufung und
	 * bestimmte Beschwerden, Verfahren im allgemeinen.
	 */
	public static final GebuehrenSatzTatbestand KV1220 = new GebuehrenSatzTatbestand(
			"Verfahren im Allgemeinen (Berufung), Ziff. 1220 KV GKG", 4.0,
			GerichtsGebuehrenTabelle.class);
	/**
	 * Der double mit dem Namen KV_1221 speichert Nr. 1221 KV-GKG. Berufung und
	 * bestimmte Beschwerden. Beendigung des gesamten Verfahrens durch
	 * Zurücknahme des Rechtsmittels, der Klage oder des Antrags, bevor die
	 * Schrift zur Begründung des Rechtsmittels bei Gericht eingegangen ist: Die
	 * Gebühr 1220 ermäßigt sich auf ......... 1,0. Erledigungserklärungen nach
	 * § 91a ZPO stehen der Zurücknahme gleich, wenn keine Entscheidung über die
	 * Kosten ergeht oder die Entscheidung einer zuvor mitgeteilten Einigung der
	 * Parteien über die Kostentragung oder der Kostenübernahmeerklärung einer
	 * Partei folgt.
	 */
	public static final GebuehrenSatzTatbestand KV1221 = new GebuehrenSatzTatbestand(
			"Verfahren im Allgemeinen (Berufung, ermäßigt), Ziff. 1221 KV GKG",
			1.0, GerichtsGebuehrenTabelle.class);
	/**
	 * Der double mit dem Namen KV_1222 speichert Nr. 1222 KV-GKG. Berufung und
	 * bestimmte Beschwerden. Beendigung des gesamten Verfahrens, wenn nicht
	 * Nummer 1221 anzuwenden ist, durch
	 * 
	 * 1. Zurücknahme des Rechtsmittels, der Klage oder des Antrags
	 * 
	 * a) vor dem Schluss der mündlichen Verhandlung, b) in den Fällen des § 128
	 * Abs. 2 ZPO vor dem Zeitpunkt, der dem Schluss der mündlichen Verhandlung
	 * entspricht,
	 * 
	 * 2. Anerkenntnisurteil, Verzichtsurteil oder Urteil, das nach § 313a Abs.
	 * 2 ZPO keinen Tatbestand und keine Entscheidungsgründe enthält, 3.
	 * gerichtlichen Vergleich oder 4. Erledigungserklärungen nach § 91a ZPO,
	 * wenn keine Entscheidung über die Kosten ergeht oder die Entscheidung
	 * einer zuvor mitgeteilten Einigung der Parteien über die Kostentragung
	 * oder der Kostenübernahmeerklärung einer Partei folgt,
	 * 
	 * es sei denn, dass bereits ein anderes als eines der in Nummer 2 genannten
	 * Urteile, eine Entscheidung über einen Antrag auf Erlass einer
	 * Sicherungsanordnung oder ein Beschluss in der Hauptsache vorausgegangen
	 * ist:
	 */
	public static final GebuehrenTatbestand KV1222 = new GebuehrenSatzTatbestand(
			"Verfahren im Allgemeinen (Berufung, ermäßigt), Ziff. 1222 KV GKG",
			2.0, GerichtsGebuehrenTabelle.class);
	/**
	 * Der double mit dem Namen KV_1223 speichert Nr. 1223 KV-GKG. Berufung und
	 * bestimmte Beschwerden. Beendigung des gesamten Verfahrens durch ein
	 * Urteil, das wegen eines Verzichts der der Parteien nach § 313a Abs. 1
	 * Satz 2 ZPO keine schriftliche Begründung enthält, wenn nicht bereits ein
	 * anderes als eines der in Nummer 1222 Nr. 2 genannten Urteile, eine
	 * Entscheidung über einen Antrag auf Erlass einer Sicherungsanordnung oder
	 * ein Beschluss in der Hauptsache vorausgegangen ist: Die Gebühr 1220
	 * ermäßigt sich... Die Gebühr ermäßigt sich auch, wenn daneben
	 * Ermäßigungstatbestände nach Nummer 1222 erfüllt sind.
	 */
	public static final GebuehrenTatbestand KV1223 = new GebuehrenSatzTatbestand(
			"Verfahren im Allgemeinen (Berufung, Urteil ohne Gründe), Ziff. 1223 KV GKG",
			3.0, GerichtsGebuehrenTabelle.class);

	/**
	 * KV_1410 speichert Arrest und einstweilige Verfügung (erster Rechtszug),
	 * Ziff. 1410 KV GKG als double.
	 */
	public static final GebuehrenTatbestand KV1410 = new GebuehrenSatzTatbestand(
			"Arrest und einstweilige Verfügung (erster Rechtszug), Ziff. 1410 KV GKG",
			1.5, GerichtsGebuehrenTabelle.class);

	/**
	 * KV1411 speichert Arrest und einstweilige Verfügung (erster Rechtszug,
	 * ermäßigt), Ziff. 1411 KV GKG.
	 */
	public static final GebuehrenTatbestand KV1411 = new GebuehrenSatzTatbestand(
			"Arrest und einstweilige Verfügung (erster Rechtszug, ermäßigt), Ziff. 1411 KV GKG",
			1.0, GerichtsGebuehrenTabelle.class);

	/**
	 * KV1412 speichert Arrest und einstweilige Verfügung (Entscheidung durch
	 * Urteil), Ziff. 1412 KV GKG
	 */
	public static final GebuehrenTatbestand KV1412 = new GebuehrenSatzTatbestand(
			"Arrest und einstweilige Verfügung (Entscheidung durch Urteil), Ziff. 1412 KV GKG",
			3.0, GerichtsGebuehrenTabelle.class);

	/**
	 * KV9003 speichert Aktenversandpauschale, Ziff. 9003 KV GKG als
	 * AuslagenTatbestand.
	 */
	public static final AuslagenTatbestand KV9003 = new AuslagenTatbestand(
			"Aktenversandpauschale, Ziff. 9003 KV GKG", 12.0,
			GerichtsGebuehrenTabelle.class);

	/**
	 * KV9005 speichert Nach dem JVEG zu zahlende Beträge, Ziff. 9005 KV GKG als
	 * AuslagenTatbestand.
	 */
	public static final AuslagenTatbestand KV9005 = new AuslagenTatbestand(
			"Nach dem JVEG zu zahlende Beträge, Ziff. 9005 KV GKG",
			GerichtsGebuehrenTabelle.class);

	public static final String GESETZ = "Gerichtskostengesetz (GKG)";

	/**
	 * Der Konstruktor ruft den Konstruktor der Oberklasse mit einem grundSprung
	 * 500 EUR und einer grundHoehe von 35 EUR auf. Ein Gültigkeitszeitraum wird
	 * nicht festgelegt. Zudem initialisiert er mittels
	 * {@link eu.gronos.kostenrechner.model.gebuehren.AnwaltsGebuehrenTabelle#fuelleTabelle()}
	 * die Gebührentabelle.
	 * 
	 * @see eu.gronos.kostenrechner.model.gebuehren.GebuehrenTabelle#GebuehrenTabelle(long,long,Calendar,Calendar)
	 */
	public GerichtsGebuehrenTabelle() {
		super();
		InputStream stream = getClass().getClassLoader().getResourceAsStream(
				"resources/gkg.xml");
		setTabelle(readTabelleFromStream(stream));
		// super(new GebuehrenTabelle(500L, 35L, null, null));
		// fuelleTabelle();
		// setTabelle(fuelleTabelle());
	}

	/**
	 * Der Konstruktor ruft den Konstruktor der Oberklasse auf. Zudem
	 * initialisiert er mittels
	 * {@link GebuehrenTabelle#readTabelleFromStream(InputStream)} und
	 * {@link #setTabelle(ArrayList)} mittels XML-Datei und {@link JAXB} die
	 * Werte der Tabelle.
	 * 
	 * @param stream
	 *            ein {@link InputStream} von einer XML-Datei für {@link JAXB}
	 */
	public GerichtsGebuehrenTabelle(InputStream stream) {
		super();
		setTabelle(readTabelleFromStream(stream));
	}

	/*
	 * Füllt die {@link GebuehrenTabelle#getTabelle() tabelle} der Oberklasse
	 * mit allen nötigen Werten. Wird vom Konstruktor aufgerufen.
	 * 
	 * @return die gefüllte {@link ArrayList}&lt;{@link GebuehrenZeile}&gt;
	 * 
	 * @see eu.gronos.kostenrechner.GebuehrenTabelle#fuelleTabelle()
	 */
	// @Override
	// private ArrayList<GebuehrenZeile> fuelleTabelle() {
	// ArrayList<GebuehrenZeile> gkg = new ArrayList<GebuehrenZeile>();
	// gkg.add(new GebuehrenGrundZeile(500L, 35L, new GregorianCalendar(2013,
	// Calendar.AUGUST, 1), null, GESETZ));
	// gkg.add(new GebuehrenZeile(2000, 500, 18));
	// gkg.add(new GebuehrenZeile(10000, 1000, 19));
	// gkg.add(new GebuehrenZeile(25000, 3000, 26));
	// gkg.add(new GebuehrenZeile(50000, 5000, 35));
	// gkg.add(new GebuehrenZeile(200000, 15000, 120));
	// gkg.add(new GebuehrenZeile(500000, 30000, 179));
	// // § 39 Abs. 2 GKG: Höchstes 30.000.000 EUR
	// gkg.add(new GebuehrenZeile(30000000, 50000, 180));
	// return gkg;
	// }

	/**
	 * Die Methode getAllGebuehrenTatbestaende dient dazu, alle vorbereiteten
	 * Instanzen von GebuehrenTatbestand, die zu der GebuehrenTabelle gehören,
	 * zu bekommen.
	 * 
	 * @return ein Array GebuehrenTatbestand[] mit den Tatbeständen.
	 */
	public static GebuehrenTatbestand[] getAllGebuehrenTatbestaende() {
		return new GebuehrenTatbestand[] { KV1100, KV1210, KV1211, KV1212,
				KV1213, KV1220, KV1221, KV1222, KV1223, KV1410, KV1411, KV1412,
				KV9003, KV9005 };
	}

	// super.gebuehrenTabelle =
	// protected ArrayList<GebuehrenZeile> fuelleTabelle() {
	// ArrayList<GebuehrenZeile> gkg;
	// gkg = new ArrayList<GebuehrenZeile>();
	// return gkg;
}
