/**
 * BeteiligtenHinzufuegenDialog.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.baumbach;

import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligter;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Dialog zum Hinzufügen eines (einfachen) BaumbachBeteiligten
 * 
 */
public class BeteiligtenHinzufuegenDialog extends HinzufuegenDialog<BaumbachBeteiligter> {

	/**
	 * Guckt bei Klägern, ob sie an der Widwrklage beteiligt sind und gibt
	 * dementsprechend die Textfelder frei.
	 *
	 * @author Peter Schuster (setrok)
	 * @date 11 Apr 2019
	 *
	 */
	public class WiderklageVeraenderbarkeitsHorcher implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent e) {
			adjustTextfieldEditability(beteiligtenTyp, cbAnWiderklageBeteiligt);
		}
	}

	/**
	 * Klasse dient dazu, adjustWiderklageCheckBox aufzurufen, wenn ein anderes
	 * Geschlecht für den Beteiligten ausgewählt wird. So kann der Text der Checkbox
	 * an den Genus der Auswahl angepasst werden.
	 *
	 * @author Peter Schuster (setrok)
	 * @date 11 Apr 2019
	 *
	 */
	public class WiderklageGenusItemHorcher implements ItemListener {
		/**
		 * Die Methode itemStateChanged dient dazu, adjustWiderklageCheckBox aufzurufen,
		 * wenn ein anderes Geschlecht für den Beteiligten ausgewählt wird. So kann der
		 * Text der Checkbox an den Genus der Auswahl angepasst werden.
		 * 
		 * The listener interface for receiving item events. The class that is
		 * interested in processing an item event implements this interface. The object
		 * created with that class is then registered with a component using the
		 * component's addItemListener method. When an item-selection event occurs, the
		 * listener object's itemStateChanged method is invoked.
		 * 
		 * @param ie
		 * 
		 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
		 */
		@Override
		public void itemStateChanged(ItemEvent ie) {
			// TODO: angeblich sind ActionListener besser für JComboBox
			adjustWiderklageCheckBox(beteiligtenTyp, cbAnWiderklageBeteiligt);
		}
	}

	private static final long serialVersionUID = 2242125254034647717L;
	private JComboBox<Beteiligter> cmbTypGenusNumerus;
	private JCheckBox cbAnWiderklageBeteiligt;
	private JFormattedTextField ftfStreitwert;
	private JFormattedTextField ftfUnterliegen;
	private final int beteiligtenTyp;
	private final boolean widerKlageAnzeigen;
	private BaumbachBeteiligter beteiligter;
	private JLabel lblStreitwert;
	private JLabel lblUnterliegen;

	/**
	 * @return beteiligtenTyp gibt den beteiligtenTyp als int zurück.
	 */
	int getBeteiligtenTyp() {
		return beteiligtenTyp;
	}

	/**
	 * Create the dialog.
	 * 
	 * @param owner              the JFrame from which the dialog is displayed
	 * @param beteiligter        ein BaumbachBeteiligter mit den voreingestellten
	 *                           Werten für den Beteiligtentyp, Genus, Numerus,
	 *                           Streitwert, Unterliegen und ob er an der Widerklage
	 *                           beteiligt ist.
	 * @param widerKlageAnzeigen wenn true, werden zusätzliche Felder für die
	 *                           Widerklage angezeigt
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#HinzufuegenDialog(JFrame,String)
	 * @see javax.swing.JDialog#JDialog(Frame,String,boolean)
	 */
	public BeteiligtenHinzufuegenDialog(JFrame owner, BaumbachBeteiligter beteiligter, boolean widerKlageAnzeigen) {
		super(owner, String.format("%s hinzufügen", beteiligter.kurzBezeichner(Beteiligter.AKKUSATIV)));
		this.widerKlageAnzeigen = widerKlageAnzeigen;
		this.beteiligter = beteiligter;
		this.beteiligtenTyp = beteiligter.getTyp();
	}

	/**
	 * Methode zum Plausis checken, wird vom OK-Button aufgerufen.
	 * 
	 * @return true, wenn alle Plausis eingehalten werden
	 */
	@Override
	public boolean parseEingabe() {
		double streitwert = 0.0;
		double unterliegen = 0.0;
		try {
			streitwert = doubleAusFormatter(ftfStreitwert, lblStreitwert.getText());// Double.parseDouble(ftfStreitwert.getText());
			unterliegen = doubleAusFormatter(ftfUnterliegen, lblUnterliegen.getText());// Double.parseDouble(ftfUnterliegen.getText());
			if (streitwert < unterliegen)
				throw new NumberFormatException(HinzufuegenDialog.FEHLER_BT_UNTERLIEGEN_GT_STREITWERT);
		} catch (NumberFormatException nfe) {
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, nfe.getLocalizedMessage(), nfe);
			// nfe.printStackTrace();
			return false;
		}
		return true;
	}

	/**
	 * Die Methode füllt den Inhaltsbereich des Dialogs mit den Eingabe- und
	 * Auswahlkomponenten
	 * 
	 */
	@Override
	protected void fuelleContentPane() {
		{
			JLabel lblBeteiligtentyp = new JLabel("Beteiligtentyp");
			contentPanel.add(lblBeteiligtentyp,
					GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));
			cmbTypGenusNumerus = fillBeteiligtenCombo(beteiligtenTyp);
			cmbTypGenusNumerus.setSelectedIndex(beteiligter.getGenusNumerus() > Beteiligter.PLURAL
					? beteiligter.getGenusNumerus() - Beteiligter.PLURAL
					: beteiligter.getGenusNumerus());
			// lblBeteiligtentyp.setLabelFor(cmbTypGenusNumerus);
			// lblBeteiligtentyp.setDisplayedMnemonic(KeyEvent.VK_B);
			setMnemonicLabelFor(lblBeteiligtentyp, cmbTypGenusNumerus, KeyEvent.VK_B);
			contentPanel.add(cmbTypGenusNumerus,
					GitterBeutelBeschraenkungen.getInstance(1, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));
			ItemListener itemListener = new WiderklageGenusItemHorcher();
			cmbTypGenusNumerus.addItemListener(itemListener);
		}
		{
			cbAnWiderklageBeteiligt = new JCheckBox();
			cbAnWiderklageBeteiligt.setSelected(beteiligter.isAnWiderklageBeteiligt());
			cbAnWiderklageBeteiligt.setVisible(widerKlageAnzeigen);
			cbAnWiderklageBeteiligt.addChangeListener(new WiderklageVeraenderbarkeitsHorcher());
			cbAnWiderklageBeteiligt.setMnemonic(KeyEvent.VK_W);
			contentPanel.add(cbAnWiderklageBeteiligt,
					GitterBeutelBeschraenkungen.getInstance(2, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));
			adjustWiderklageCheckBox(beteiligtenTyp, cbAnWiderklageBeteiligt);
		}
		{
			lblStreitwert = new JLabel("Höhe der Inanspruchnahme / Klageantrag");
			contentPanel.add(lblStreitwert,
					GitterBeutelBeschraenkungen.getInstance(0, 2, 1, 1, GridBagConstraints.HORIZONTAL, false));
			ftfStreitwert = new JFormattedTextField(formatter);
			ftfStreitwert.setText(Double.toString(beteiligter.getStreitwert()));
			setEnterAction(ftfStreitwert, getOkAction());
			if (!widerKlageAnzeigen && beteiligtenTyp == Beteiligter.KLAEGER) {
				ftfStreitwert.setText("0");
				lblStreitwert.setVisible(false);
				ftfStreitwert.setVisible(false);
			}
			setMnemonicLabelFor(lblStreitwert, ftfStreitwert, KeyEvent.VK_I);
			// lblStreitwert.setLabelFor(ftfStreitwert);
			// lblStreitwert.setDisplayedMnemonic(KeyEvent.VK_I);
			contentPanel.add(ftfStreitwert,
					GitterBeutelBeschraenkungen.getInstance(1, 2, 2, 1, GridBagConstraints.HORIZONTAL, false));
		}
		{
			lblUnterliegen = new JLabel("Höhe der Verurteilung / darüber hinaus");
			contentPanel.add(lblUnterliegen,
					GitterBeutelBeschraenkungen.getInstance(0, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));
			ftfUnterliegen = new JFormattedTextField(formatter);
			ftfUnterliegen.setText(Double.toString(beteiligter.getUnterliegen()));
			setEnterAction(ftfUnterliegen, getOkAction());
			if (!widerKlageAnzeigen && beteiligtenTyp == Beteiligter.KLAEGER) {
				ftfUnterliegen.setText("0");
				lblUnterliegen.setVisible(false);
				ftfUnterliegen.setVisible(false);
			}
			// lblUnterliegen.setLabelFor(ftfUnterliegen);
			// lblUnterliegen.setDisplayedMnemonic(KeyEvent.VK_V);
			setMnemonicLabelFor(lblUnterliegen, ftfUnterliegen, KeyEvent.VK_V);
			contentPanel.add(ftfUnterliegen,
					GitterBeutelBeschraenkungen.getInstance(1, 3, 2, 1, GridBagConstraints.HORIZONTAL, false));
			adjustTextfieldEditability(beteiligtenTyp, cbAnWiderklageBeteiligt);
		}
	}

	@Override
	protected BaumbachBeteiligter baueRueckgabewert() {
		return new BaumbachBeteiligter(beteiligtenTyp, cmbTypGenusNumerus.getSelectedIndex(),
				doubleAusFormatter(ftfStreitwert, lblStreitwert.getText()),
				doubleAusFormatter(ftfUnterliegen, lblUnterliegen.getText()),
				widerKlageAnzeigen ? cbAnWiderklageBeteiligt.isSelected() : false);
	}

	/**
	 * Die Methode fillBeteiligtenCombo dient dazu, eine JComboBox mit Beteiligten
	 * eines Typs zu füllen
	 * 
	 * @param beteiligtenTyp
	 * @return
	 */
	private JComboBox<Beteiligter> fillBeteiligtenCombo(int beteiligtenTyp) {
		Beteiligter[] beteiligtenAuswahlListe = BaumbachBeteiligter.getBeteiligtenAuswahlListe(beteiligtenTyp, false);
		JComboBox<Beteiligter> jComboBox = new JComboBox<Beteiligter>(beteiligtenAuswahlListe);
		jComboBox.setEnabled(true);
		jComboBox.setEditable(false);
		jComboBox.setRenderer(StreitgenossenPanel.BETEILIGTEN_RENDERER_OHNE_LFDNR);
		return jComboBox;
	}

	/**
	 * Die Methode adjustWiderklageCheckBox dient dazu, den Text der JCheckBox
	 * chckbxAnWiderklageBeteiligt an den BeteiligtenTyp anzupassen ("und
	 * Widerkläger/Widerbeklagter")
	 * 
	 * @param beteiligtenTyp              der hinzuzufügende beteiligtenTyp
	 * @param chckbxAnWiderklageBeteiligt der Verweis auf die JCheckBox
	 */
	private void adjustWiderklageCheckBox(int beteiligtenTyp, JCheckBox chckbxAnWiderklageBeteiligt) {
		final String[] widerBeklagtenBeugung = new String[] { "Widerbeklagter", "Widerbeklagte" };
		final String[] widerKlaegerBeugung = new String[] { "Widerkläger", "Widerklägerin" };
		int genusNumerus = 0;
		if (cmbTypGenusNumerus != null)
			genusNumerus = cmbTypGenusNumerus.getSelectedIndex();
		switch (beteiligtenTyp) {
		case Beteiligter.DRITTWIDERBEKLAGTE:
			chckbxAnWiderklageBeteiligt.setSelected(true);
			chckbxAnWiderklageBeteiligt.setEnabled(false);
			// jetzt kommt ein fall-through
		case Beteiligter.KLAEGER:
			chckbxAnWiderklageBeteiligt.setText("und " + widerBeklagtenBeugung[genusNumerus]);
			break;
		case Beteiligter.BEKLAGTE:
			chckbxAnWiderklageBeteiligt.setText("und " + widerKlaegerBeugung[genusNumerus]);
			break;
		default:
			chckbxAnWiderklageBeteiligt.setText("und Widerkläger/Widerbeklagter");
		}
	}

	/**
	 * Die Methode sperrt die Eingabe-Textfelder bei Klägern, sofern die Widerklage
	 * nicht aktiviert ist
	 * 
	 * @param beteiligtenTyp          eine Konstante Beteiligter.KLAEGER,
	 *                                Beteiligter.DRITTWIDERBEKLAGTE,
	 *                                Beteiligter.BEKLAGTE
	 * @param cbAnWiderklageBeteiligt die JCheckBox
	 */
	protected void adjustTextfieldEditability(int beteiligtenTyp, JCheckBox cbAnWiderklageBeteiligt) {
		if (!widerKlageAnzeigen || ftfStreitwert == null || ftfUnterliegen == null)
			return;
		if (beteiligtenTyp == Beteiligter.KLAEGER) {
			if (cbAnWiderklageBeteiligt.isSelected()) {
				ftfStreitwert.setEnabled(true);
				ftfUnterliegen.setEnabled(true);
			} else {
				ftfStreitwert.setEnabled(false);
				ftfStreitwert.setText("0");
				ftfUnterliegen.setEnabled(false);
				ftfUnterliegen.setText("0");
			}
		}
	}

}
